@@ -122,7 +122,7 @@
 
 /*----------------------------------------------------------------------------*/
 /**
- * @defgroup muse_idp     ESO science data product format support
+ * @defgroup muse_idp          ESO science data product format support
  */
 /*----------------------------------------------------------------------------*/
 
@@ -206,6 +206,12 @@ static const double kMuseResponseRef = 41.2;
 /* Typical error estimated from unresolved sources [arcsec] */
 static const double kMuseExtraErrorIQE = 0.3;
 
+/* Typical values [arcsec] for the effective spatial resolution */
+/* and its uncertainty, given by the median and the standard    */
+/* deviation of previous measurements.                          */
+static const double kMuseSkyRes      = 0.853823;
+static const double kMuseSkyResError = 0.495547;
+
 
 /* Unit conversion factors */
 
@@ -816,13 +822,14 @@ muse_idp_properties_collect(muse_processing *aProcessing,
         }
       }
 
-      if (!cpl_errorstate_is_equal(status)) {
+      if (!cpl_errorstate_is_equal(_status)) {
         if (cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND) {
           cpl_msg_debug(__func__, "Keyword \"INS.MODE\" is missing in '%s'!",
                         filename);
         }
         cpl_msg_error(__func__, "Cannot determine pixel scales of exposure '%s'",
                       filename);
+        cpl_propertylist_delete(_header);
         break;
       }
 
@@ -832,26 +839,41 @@ muse_idp_properties_collect(muse_processing *aProcessing,
 
       /* Get image quality (at the reference wavelength) and the associated *
        * error for this exposure                                            */
-      double fwhmIA    = muse_pfits_get_ia_fwhmlin(_header);
-      double zstart    = muse_pfits_get_airmass_start(_header);
-      double zend      = muse_pfits_get_airmass_end(_header);
-
-      double iqe = muse_idp_compute_iqe(kMuseLambdaRef, fwhmIA,
-                                        0.5 * (zstart + zend));
-
-      properties->skyres += exptime * iqe;
+      double zstart = muse_pfits_get_airmass_start(_header);
+      double zend   = muse_pfits_get_airmass_end(_header);
 
       _status = cpl_errorstate_get();
       double fwhmstart = muse_pfits_get_fwhm_start(_header);
       double fwhmend   = muse_pfits_get_fwhm_end(_header);
 
-      if (!cpl_errorstate_is_equal(status)) {
+      if (!cpl_errorstate_is_equal(_status)) {
           cpl_msg_debug(__func__, "Missing or invalid seeing information "
                   "(\"TEL.AMBI.FWHM.START\", \"TEL.AMBI.FWHM.END\") in "
                   "exposure '%s'", filename);
           cpl_errorstate_set(_status);
       }
 
+      double iqe    = kMuseSkyRes;
+      double fwhmIA = muse_pfits_get_ia_fwhmlin(_header);
+
+      if (cpl_errorstate_is_equal(_status)) {
+        iqe = muse_idp_compute_iqe(kMuseLambdaRef, fwhmIA,
+                                   0.5 * (zstart + zend));
+      }
+      else {
+        /* Ignore the error and use the default constant instead. */
+        /* Set fwhmstart and fwhmend to 0. This forces fwhmmin to */
+        /* zero and the uncertainty of SKY_RES to become the      */
+        /* default constant.                                      */
+        cpl_msg_debug(__func__, "Missing or invalid seeing information "
+                "(\"TEL.IA.FWHMLIN\") in exposure '%s'", filename);
+        cpl_errorstate_set(_status);
+        fwhmstart = 0.;
+        fwhmend   = 0.;
+      }
+
+      properties->skyres += exptime * iqe;
+
       fwhmmin = CPL_MIN(fwhmmin, CPL_MIN(fwhmstart, fwhmend));
       fwhmmax = CPL_MAX(fwhmmax, CPL_MAX(fwhmstart, fwhmend));
 
@@ -975,7 +997,7 @@ muse_idp_properties_collect(muse_processing *aProcessing,
   } else {
       cpl_msg_warning(__func__, "Got invalid seeing information! Using an "
                       "estimate for \"SKY_RERR\"!");
-      properties->skyrerr = kMuseExtraErrorIQE;
+      properties->skyrerr = kMuseSkyResError;
   }
 
 
@@ -1180,8 +1202,8 @@ muse_idp_properties_update(cpl_propertylist *aHeader,
     unsigned int ik = 1;
     cpl_size idx;
     for (idx = 1; idx < aProperties->ncombine; ++idx) {
-      long _obid = cpl_array_get_long(aProperties->obid, idx, NULL);
-      if (obid != _obid) {
+      long _obid = cpl_array_get_long(obids, idx, NULL);
+      if (_obid != obid) {
         char *key = cpl_sprintf(KEY_OBID "%-u", ++ik);
         cpl_propertylist_update_long(aHeader, key, _obid);
         cpl_propertylist_set_comment(aHeader, key, KEY_OBID_COMMENT);
@@ -1203,8 +1225,8 @@ muse_idp_properties_update(cpl_propertylist *aHeader,
     unsigned int nprogid = 1;
     cpl_size idx;
     for (idx = 1; idx < aProperties->ncombine; ++idx) {
-      const char *_progid = cpl_array_get_string(aProperties->progid, idx);
-      if (strcmp(progid, _progid) != 0) {
+      const char *_progid = cpl_array_get_string(progids, idx);
+      if (strcmp(_progid, progid) != 0) {
         progid = _progid;
         ++nprogid;
       }
@@ -1221,8 +1243,8 @@ muse_idp_properties_update(cpl_propertylist *aHeader,
 
       unsigned int ik = 1;
       for (idx = 1; idx < aProperties->ncombine; ++idx) {
-        const char *_progid = cpl_array_get_string(aProperties->progid, idx);
-        if (strcmp(progid, _progid) != 0) {
+        const char *_progid = cpl_array_get_string(progids, idx);
+        if (strcmp(_progid, progid) != 0) {
           char *key = cpl_sprintf(KEY_PROGID "%-u", ++ik);
           cpl_propertylist_update_string(aHeader, key, _progid);
           cpl_propertylist_set_comment(aHeader, key, KEY_PROGID_COMMENT);
