@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,9 +52,9 @@
 
 /*----------------------------------------------------------------------------*/
 /**
- * @defgroup muse_postproc       Post-processing functions
+ * @defgroup muse_postproc     Post-processing functions
  *
- * This group contains functions that are common to some post-processing
+ * This module contains functions that are common to some post-processing
  * recipes.
  */
 /*----------------------------------------------------------------------------*/
@@ -106,9 +107,9 @@ muse_postproc_properties_delete(muse_postproc_properties *aProp)
     return;
   }
   cpl_table_delete(aProp->exposures);
-  cpl_table_delete(aProp->response);
+  muse_table_delete(aProp->response);
+  muse_table_delete(aProp->telluric);
   cpl_table_delete(aProp->extinction);
-  cpl_table_delete(aProp->telluric);
   cpl_propertylist_delete(aProp->wcs);
   muse_lsf_cube_delete_all(aProp->lsf_cube);
 #ifdef USE_LSF_PARAMS
@@ -394,6 +395,65 @@ muse_postproc_load_nearest(const cpl_propertylist *aHeader,
   return table;
 } /* muse_postproc_load_nearest() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @private
+  @brief    Revert correction of on-sky data with the flat-field spectrum.
+  @param    aPt         pixel table with the on-sky data
+  @param    aResponse   the response table with header
+  @return   CPL_ERROR_NONE if nothing needs to be done or on success, another
+            CPL error code on failure.
+
+  This function is a dirty trick to improve backward compatibility: if an old
+  (pre v2.0, without flat-field spectrum correction) STD_RESPONSE was passed to
+  the one of the post-processing recipes, then make the (merged) pixel table
+  compatible to that response. by reverting application of the flat-field
+  spectrum.
+  In effect, this function divides the pixel table again by the averaged
+  flat-field spectrum.
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_error_code
+muse_postproc_revert_ffspec_maybe(muse_pixtable *aPt,
+                                  const muse_table *aResponse)
+{
+  cpl_ensure_code(aPt && aPt->header, CPL_ERROR_NULL_INPUT);
+  if (!aResponse) {
+    return CPL_ERROR_NONE;
+  }
+  /* cross-check the header of the response curve against *
+   * the flat-field spectrum status of the pixel table    */
+  cpl_boolean respff = cpl_propertylist_has(aResponse->header,
+                                            MUSE_HDR_FLUX_FFCORR),
+              ptff = cpl_propertylist_has(aPt->header, MUSE_HDR_PT_FFCORR);
+  if (respff == ptff || !ptff) {
+    /* same state for both means we don't need to do anything, same if *
+     * the pixel table was not corrected for ffspec                    */
+    return CPL_ERROR_NONE;
+  }
+
+  const char *id = "muse_postproc_process_exposure"; /* pretend to be there */
+  cpl_msg_warning(id, "Adapt pixel table to %s for backward compatibility: "
+                  "revert correction by flat-field spectrum!",
+                  MUSE_TAG_STD_RESPONSE);
+
+  /* so we need to divide again by the flat-field spectrum */
+  cpl_array *lambdas = muse_cpltable_extract_column(aPt->ffspec,
+                                                    MUSE_PIXTABLE_FFLAMBDA),
+            *ffdata = muse_cpltable_extract_column(aPt->ffspec,
+                                                   MUSE_PIXTABLE_FFDATA);
+  muse_pixtable_spectrum_apply(aPt, lambdas, ffdata,
+                               MUSE_PIXTABLE_OPERATION_DIVIDE);
+  cpl_array_unwrap(lambdas);
+  cpl_array_unwrap(ffdata);
+  cpl_table_delete(aPt->ffspec);
+  aPt->ffspec = NULL;
+  cpl_propertylist_erase(aPt->header, MUSE_HDR_PT_FFCORR);
+  cpl_msg_info(id, "Pixel table now convolved with flat-field spectrum again, "
+               "removed %s keyword from header.", MUSE_HDR_PT_FFCORR);
+  return CPL_ERROR_NONE;
+} /* muse_postproc_revert_ffspec_maybe() */
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Merge and process pixel tables from one exposure.
@@ -462,8 +522,11 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
   /* erase pre-existing QC parameters */
   cpl_propertylist_erase_regexp(pt->header, "ESO QC ", 0);
 
-  /* XXX try to recognize a lab exposure by missing RA/DEC or weird   *
-   *     airmass; in the future this should be a template-based check */
+  /* see if we need to revert the flat-field spectrum correction, *
+   * to match pixel table and response curve                      */
+  muse_postproc_revert_ffspec_maybe(pt, aProp->response);
+
+  /* try to recognize a lab exposure by missing RA/DEC or weird airmass */
   cpl_boolean labdata = CPL_FALSE;
   cpl_errorstate prestate = cpl_errorstate_get();
   double airmass = muse_astro_airmass(pt->header);
@@ -486,14 +549,15 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     /* check and possibly correct the DAR quality, if requested to do so */
     if (aProp->darcheck != MUSE_POSTPROC_DARCHECK_NONE) {
       cpl_boolean dorefine = aProp->darcheck == MUSE_POSTPROC_DARCHECK_CORRECT;
-      cpl_msg_info(__func__, "Carrying out DAR %s", dorefine ? "correction" : "check");
       double maxshift = 0;
       rc = muse_dar_check(pt, &maxshift, dorefine, NULL);
       if (rc != CPL_ERROR_NONE) {
-        cpl_msg_warning(__func__, "Maximum detected shift %f\'\' (rc = %d: %s)",
-                        maxshift, rc, cpl_error_get_message());
+        cpl_msg_warning(__func__, "Maximum %s shift %f\'\' (rc = %d: %s)",
+                        dorefine ? "corrected" : "detected", maxshift, rc,
+                        cpl_error_get_message());
       } else {
-        cpl_msg_info(__func__, "Maximum detected shift %f\'\'", maxshift);
+        cpl_msg_info(__func__, "Maximum %s shift %f\'\'",
+                     dorefine ? "corrected" : "detected", maxshift);
       }
     } /* if DAR checking */
   } /* if not labdata but WFM */
@@ -503,8 +567,17 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     if (aProp->response) {
       rc = muse_flux_calibrate(pt, aProp->response, aProp->extinction,
                                aProp->telluric);
-      cpl_msg_debug(__func__, "Flux calibration returned rc=%d: %s", rc,
-                    rc != CPL_ERROR_NONE ? cpl_error_get_message_default(rc) : "");
+      if (rc != CPL_ERROR_NONE && rc != CPL_ERROR_UNSUPPORTED_MODE) {
+        /* for a more serious error, output a warning at least *
+         * (the data may still be usable)                      */
+        cpl_msg_warning(__func__, "Flux calibration returned rc=%d: %s", rc,
+                        cpl_error_get_message());
+      } else {
+        /* for no error or unsupported mode (an already flux         *
+         * calibrated table!) a debug message seems more appropriate */
+        cpl_msg_debug(__func__, "Flux calibration returned rc=%d: %s", rc,
+                      rc != CPL_ERROR_NONE ? cpl_error_get_message_default(rc) : "");
+      }
     } else {
       cpl_msg_info(__func__, "Skipping flux calibration (no %s curve)",
                    MUSE_TAG_STD_RESPONSE);
@@ -535,13 +608,14 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
       muse_pixtable_delete(pt);
       return NULL;
     }
-    cpl_table *fwhite = muse_table_load_filter(NULL, "white");
+    muse_table *fwhite = muse_table_load_filter(NULL, "white");
     muse_image *whitelight = muse_datacube_collapse(cube, fwhite);
+    muse_table_delete(fwhite);
     cpl_msg_indent_less();
     muse_mask *sky_mask = muse_sky_create_skymask(whitelight,
                                                   aProp->skymodel_params.ignore,
                                                   aProp->skymodel_params.fraction,
-            /* XXX hardcode prefix for now... */  "ESO QC SCIPOST");
+              /* hardcode only sensible prefix */ "ESO QC SCIPOST");
 
     /* Apply mask and create spectrum, possibly with one CR-reject iteration */
     cpl_table_select_all(sky_pt->table);
@@ -619,7 +693,6 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
       cpl_table_delete(spectrum);
     }
     muse_image_delete(whitelight);
-    cpl_table_delete(fwhite);
     muse_resampling_params_delete(params);
     muse_datacube_delete(cube);
     muse_pixtable_delete(sky_pt);
@@ -711,6 +784,7 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     cpl_table_delete(reftable);
     rc = muse_flux_get_response_table(flux, aProp->smooth);
     rc = muse_flux_get_telluric_table(flux);
+    rc = muse_flux_compute_qc(flux);
 
     if (!cpl_errorstate_is_equal(prestate)) {
       cpl_msg_warning(__func__, "The following errors happened while computing "
@@ -936,8 +1010,8 @@ muse_postproc_cube_resample_and_collapse(muse_processing *aProcessing,
   int i, ipos = 0, nfilters = cpl_array_get_size(filters);
   for (i = 0; i < nfilters; i++) {
     /* try to find and load the filter from a table */
-    cpl_table *filter = muse_table_load_filter(aProcessing,
-                                               cpl_array_get_string(filters, i));
+    muse_table *filter = muse_table_load_filter(aProcessing,
+                                                cpl_array_get_string(filters, i));
     if (!filter) {
       continue;
     }
@@ -958,7 +1032,7 @@ muse_postproc_cube_resample_and_collapse(muse_processing *aProcessing,
         fov = muse_datacube_collapse(cube, filter);
       }
     }
-    cpl_table_delete(filter);
+    muse_table_delete(filter);
 
     if (aFormat == MUSE_CUBE_TYPE_EURO3D_X || aFormat == MUSE_CUBE_TYPE_FITS_X) {
       if (!((muse_datacube *)cube)->recimages) {
@@ -1043,7 +1117,7 @@ muse_postproc_cube_resample_and_collapse(muse_processing *aProcessing,
   The prefix of the generated QC1 FITS keywords is set depending on the recipe
   name in the aProcessing structure.
 
-  XXX Part of this is copied in the image_fwhm tool.
+  @note Part of this is copied in the image_fwhm tool.
 
   @algorithm This function detects objects in the central wavelength plane
              of the input datacube, derives the FWHM for each source in x-
