@@ -29,16 +29,16 @@
 
 #include "muse_flux.h"
 #include "muse_lsf.h"
+#include "muse_lsf_params.h"
 #include "muse_pixtable.h"
 #include "muse_processing.h"
 #include "muse_resampling.h"
 #include "muse_rvcorrect.h"
 #include "muse_sky.h"
+#include "muse_table.h"
 #include "muse_wcs.h"
 #include "muse_xcombine.h"
 
-#include "muse_lsf_params.h"
-
 /*----------------------------------------------------------------------------*
  *                              Defines                                       *
  *----------------------------------------------------------------------------*/
@@ -96,9 +96,9 @@ typedef struct {
   double lambdaref; /**< reference wavelength for DAR correction */
   muse_postproc_darcheck darcheck; /**< if to check or even correct DAR */
   muse_rvcorrect_type rvtype; /**< type of radial velocity correction to apply */
-  cpl_table *response; /**< flux response table */
+  muse_table *response; /**< flux response table */
+  muse_table *telluric; /**< the telluric correction table */
   cpl_table *extinction; /**< the extinction curve table */
-  cpl_table *telluric; /**< the telluric correction table */
   cpl_propertylist *wcs; /**< FITS headers with astrometry calibration */
   muse_postproc_skymethod skymethod; /**< the sky subtraction method */
   cpl_table *sky_lines; /**< the sky lines table */
