@@ -584,9 +584,9 @@ muse_wcs_locate_sources(muse_pixtable *aPixtable, float aSigma,
   cube->recnames = cpl_array_new(2, CPL_TYPE_STRING);
   /* create a white-light image; this is unused here but should *
    * be saved as first extension in the output cube             */
-  cpl_table *fwhite = muse_table_load_filter(NULL, "white");
+  muse_table *fwhite = muse_table_load_filter(NULL, "white");
   muse_image *white = muse_datacube_collapse(cube, fwhite);
-  cpl_table_delete(fwhite);
+  muse_table_delete(fwhite);
   muse_imagelist_set(cube->recimages, white, 0);
   cpl_array_set_string(cube->recnames, 0, "white");
   /* append this median-combined detection image also as *
