@@ -84,9 +84,12 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
     pixtable = NULL;
   }
 
-  cpl_table *response = muse_table_load(aProcessing, MUSE_TAG_STD_RESPONSE, 0);
-  cpl_table *extinction = muse_table_load(aProcessing, MUSE_TAG_EXTINCT_TABLE, 0);
-  cpl_table *telluric = muse_table_load(aProcessing, MUSE_TAG_STD_TELLURIC, 0);
+  muse_table *response = muse_processing_load_table(aProcessing,
+                                                    MUSE_TAG_STD_RESPONSE, 0),
+             *telluric = muse_processing_load_table(aProcessing,
+                                                    MUSE_TAG_STD_TELLURIC, 0);
+  cpl_table *extinction = muse_processing_load_ctable(aProcessing,
+                                                      MUSE_TAG_EXTINCT_TABLE, 0);
 
   if ((pixtable != NULL) && (response != NULL)) {
     if (muse_pixtable_is_fluxcal(pixtable) == CPL_TRUE) {
@@ -97,7 +100,8 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
       muse_pixtable_delete(pixtable);
       pixtable = NULL;
     } else {
-      cpl_error_code rc = muse_flux_calibrate(pixtable, response, extinction, telluric);
+      cpl_error_code rc = muse_flux_calibrate(pixtable, response, extinction,
+                                              telluric);
       if (rc != CPL_ERROR_NONE) {
         cpl_msg_error(__func__, "while muse_flux_calibrate");
         muse_pixtable_delete(pixtable);
@@ -106,9 +110,9 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
     }
   }
 
-  cpl_table_delete(response);
+  muse_table_delete(response);
+  muse_table_delete(telluric);
   cpl_table_delete(extinction);
-  cpl_table_delete(telluric);
 
   if ((pixtable != NULL) && (muse_pixtable_is_fluxcal(pixtable) != CPL_TRUE)) {
     cpl_msg_error(__func__, "Pixel table not flux calibrated");
@@ -166,7 +170,7 @@ muse_create_sky_whitelight_image(muse_pixtable *aPixtable, double aCRSigma)
     return NULL;
   }
   muse_image *image = NULL;
-  cpl_table *fwhite = muse_table_load_filter(NULL, "white");
+  muse_table *fwhite = muse_table_load_filter(NULL, "white");
   if (usegrid) {
     params->method = MUSE_RESAMPLE_WEIGHTED_DRIZZLE;
     image = muse_resampling_collapse_pixgrid(aPixtable, grid,
@@ -177,7 +181,7 @@ muse_create_sky_whitelight_image(muse_pixtable *aPixtable, double aCRSigma)
   muse_resampling_params_delete(params);
   muse_pixgrid_delete(grid);
   muse_datacube_delete(cube);
-  cpl_table_delete(fwhite);
+  muse_table_delete(fwhite);
   return image;
 } /* muse_create_sky_whitelight_image() */
 
@@ -221,7 +225,7 @@ muse_create_sky_compute(muse_processing *aProcessing,
      - select pixtable rows containing sky
      - create spectrum from selected rows
   */
-  muse_mask *smask = muse_processing_mask_load(aProcessing, MUSE_TAG_SKY_MASK);
+  muse_mask *smask = muse_processing_load_mask(aProcessing, MUSE_TAG_SKY_MASK);
   if (smask) { /* apply existing mask on input pixel table */
     cpl_table_select_all(pixtable->table);
     muse_pixtable_and_selected_mask(pixtable, smask);
