@@ -24,9 +24,71 @@
 #include <math.h>
 #include <string.h>
 
+extern const muse_cpltable_def muse_filtertable_def[];
+
+muse_table *
+muse_test_flux_cousins_r(void)
+{
+  cpl_table *table = muse_cpltable_new(muse_filtertable_def, 24);
+  cpl_table_set_double(table, "lambda",  0, 5500.);
+  cpl_table_set_double(table, "lambda",  1, 5600.);
+  cpl_table_set_double(table, "lambda",  2, 5700.);
+  cpl_table_set_double(table, "lambda",  3, 5800.);
+  cpl_table_set_double(table, "lambda",  4, 5900.);
+  cpl_table_set_double(table, "lambda",  5, 6000.);
+  cpl_table_set_double(table, "lambda",  6, 6100.);
+  cpl_table_set_double(table, "lambda",  7, 6200.);
+  cpl_table_set_double(table, "lambda",  8, 6300.);
+  cpl_table_set_double(table, "lambda",  9, 6400.);
+  cpl_table_set_double(table, "lambda", 10, 6500.);
+  cpl_table_set_double(table, "lambda", 11, 6600.);
+  cpl_table_set_double(table, "lambda", 12, 6700.);
+  cpl_table_set_double(table, "lambda", 13, 6800.);
+  cpl_table_set_double(table, "lambda", 14, 6900.);
+  cpl_table_set_double(table, "lambda", 15, 7000.);
+  cpl_table_set_double(table, "lambda", 16, 7100.);
+  cpl_table_set_double(table, "lambda", 17, 7200.);
+  cpl_table_set_double(table, "lambda", 18, 7300.);
+  cpl_table_set_double(table, "lambda", 19, 7400.);
+  cpl_table_set_double(table, "lambda", 20, 7500.);
+  cpl_table_set_double(table, "lambda", 21, 8000.);
+  cpl_table_set_double(table, "lambda", 22, 8500.);
+  cpl_table_set_double(table, "lambda", 23, 9000.);
+  cpl_table_set_double(table, "throughput",  0, 0.00E+00);
+  cpl_table_set_double(table, "throughput",  1, 2.30E-01);
+  cpl_table_set_double(table, "throughput",  2, 7.40E-01);
+  cpl_table_set_double(table, "throughput",  3, 9.10E-01);
+  cpl_table_set_double(table, "throughput",  4, 9.80E-01);
+  cpl_table_set_double(table, "throughput",  5, 1.00E+00);
+  cpl_table_set_double(table, "throughput",  6, 9.80E-01);
+  cpl_table_set_double(table, "throughput",  7, 9.60E-01);
+  cpl_table_set_double(table, "throughput",  8, 9.30E-01);
+  cpl_table_set_double(table, "throughput",  9, 9.00E-01);
+  cpl_table_set_double(table, "throughput", 10, 8.60E-01);
+  cpl_table_set_double(table, "throughput", 11, 8.10E-01);
+  cpl_table_set_double(table, "throughput", 12, 7.80E-01);
+  cpl_table_set_double(table, "throughput", 13, 7.20E-01);
+  cpl_table_set_double(table, "throughput", 14, 6.70E-01);
+  cpl_table_set_double(table, "throughput", 15, 6.10E-01);
+  cpl_table_set_double(table, "throughput", 16, 5.60E-01);
+  cpl_table_set_double(table, "throughput", 17, 5.10E-01);
+  cpl_table_set_double(table, "throughput", 18, 4.60E-01);
+  cpl_table_set_double(table, "throughput", 19, 4.00E-01);
+  cpl_table_set_double(table, "throughput", 20, 3.50E-01);
+  cpl_table_set_double(table, "throughput", 21, 1.40E-01);
+  cpl_table_set_double(table, "throughput", 22, 3.00E-02);
+  cpl_table_set_double(table, "throughput", 23, 0.00E+00);
+
+  muse_table *mt = muse_table_new();
+  mt->table = table;
+  mt->header = cpl_propertylist_new();
+  cpl_propertylist_append_string(mt->header, "EXTNAME", "Cousins_R");
+  return mt;
+} /* muse_test_flux_cousins_r() */
+
 void
-muse_test_flux_do_calibrate(muse_pixtable *aPt, cpl_table *aResponse,
-                            cpl_table *aExtinction, cpl_table *aTelluric)
+muse_test_flux_do_calibrate(muse_pixtable *aPt, muse_table *aResponse,
+                            cpl_table *aExtinction, muse_table *aTelluric)
 {
   cpl_msg_debug(__func__, "Starting muse_flux_calibrate(%s extinction, "
                 "%s telluric)...", aExtinction ? "with" : "without",
@@ -301,16 +363,19 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
   fo->intimage = intimage;
   fo->raref = muse_pfits_get_ra(cube->header);
   fo->decref = muse_pfits_get_dec(cube->header);
+  cpl_test_null(fo->reference);
   muse_flux_response_compute(fo, select, airmass, reftable,
                              NULL, exttable);
+  cpl_test_nonnull(fo->reference);
   muse_flux_get_response_table(fo, smooth);
+  muse_flux_compute_qc(fo);
   fn = cpl_sprintf("%s_STD_RESPONSE.fits", incube);
-  cpl_table_save(fo->response, cube->header, NULL, fn, CPL_IO_CREATE);
+  muse_table_save(fo->response, fn);
   cpl_msg_info(__func__, "Wrote response as \"%s\".", fn);
   cpl_free(fn);
   muse_flux_get_telluric_table(fo);
   fn = cpl_sprintf("%s_STD_TELLURIC.fits", incube);
-  cpl_table_save(fo->telluric, cube->header, NULL, fn, CPL_IO_CREATE);
+  muse_table_save(fo->telluric, fn);
   cpl_msg_info(__func__, "Wrote telluric as \"%s\".", fn);
   cpl_free(fn);
   muse_flux_object_delete(fo);
@@ -336,6 +401,8 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
     muse_flux_response_compute
     muse_flux_get_response_table
     muse_flux_get_telluric_table
+    muse_flux_compute_qc
+    muse_flux_compute_qc_zp
     muse_flux_calibrate
  */
 /*----------------------------------------------------------------------------*/
@@ -418,8 +485,9 @@ int main(int argc, char **argv)
   state = cpl_errorstate_get();
   muse_pixtable *pt = muse_pixtable_load(BASEFILENAME"_pixtable_gd71sm.fits");
   cpl_test_nonnull(pt);
-  muse_pixtable *ptdupe = muse_pixtable_duplicate(pt);
   cpl_errorstate_set(state); /* hide error from missing DQ and Origin columns */
+  cpl_propertylist_append_string(pt->header, "ESO OBS TARG NAME", "GD71"); /* for QC */
+  muse_pixtable *ptdupe = muse_pixtable_duplicate(pt);
   /* add (clean) DQ and (empty) origin columns back *
    * which were removed to save space               */
   cpl_table_new_column(pt->table, MUSE_PIXTABLE_DQ, CPL_TYPE_INT);
@@ -559,36 +627,155 @@ int main(int argc, char **argv)
   rc = muse_flux_response_compute(fluxobj, MUSE_FLUX_SELECT_BRIGHTEST, airmass,
                                   ref, NULL, extinction);
   cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+  cpl_test_nonnull(fluxobj->reference);
   cpl_test_nonnull(fluxobj->sensitivity);
+
+  /* create a filter table for Cousins_R for testing */
+  muse_table *cousinsr = muse_test_flux_cousins_r();
+
+  /* test muse_flux_compute_qc(), in this case just that it runs    *
+   * through (and manually, that it prints the QC parameter values) */
+  state = cpl_errorstate_get();
+  rc = muse_flux_compute_qc(fluxobj);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  /* test failures of muse_flux_compute_qc() */
+  state = cpl_errorstate_get();
+  rc = muse_flux_compute_qc(NULL);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_table *stmp = fluxobj->sensitivity;
+  fluxobj->sensitivity = NULL;
+  rc = muse_flux_compute_qc(fluxobj); /* neither sensitivity nor response */
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, "Cousins_R");
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  fluxobj->sensitivity = stmp;
+
+  /* check muse_flux_get_response_table() */
   rc = muse_flux_get_response_table(fluxobj, MUSE_FLUX_SMOOTH_MEDIAN);
   cpl_test_nonnull(fluxobj->response);
-  cpl_test(cpl_table_has_column(fluxobj->response, "lambda") &&
-           cpl_table_has_column(fluxobj->response, "response") &&
-           cpl_table_has_column(fluxobj->response, "resperr"));
-  cpl_test(cpl_table_get_nrow(fluxobj->response)
+  cpl_test_nonnull(fluxobj->response->table);
+  cpl_test_nonnull(fluxobj->response->header);
+  cpl_test(cpl_table_has_column(fluxobj->response->table, "lambda") &&
+           cpl_table_has_column(fluxobj->response->table, "response") &&
+           cpl_table_has_column(fluxobj->response->table, "resperr"));
+  cpl_test(cpl_table_get_nrow(fluxobj->response->table)
            == cpl_table_get_nrow(fluxobj->sensitivity));
   /* check resulting responses at three wavelengths, values were read *
    * off by eye from the sensitivity plots around these wavelengths   */
-  double r1 = muse_flux_response_interpolate(fluxobj->response, 5000., NULL,
+  double r1 = muse_flux_response_interpolate(fluxobj->response->table, 5000., NULL,
                                              MUSE_FLUX_RESP_FLUX),
-         r2 = muse_flux_response_interpolate(fluxobj->response, 7000., NULL,
+         r2 = muse_flux_response_interpolate(fluxobj->response->table, 7000., NULL,
                                              MUSE_FLUX_RESP_FLUX),
-         r3 = muse_flux_response_interpolate(fluxobj->response, 9000., NULL,
+         r3 = muse_flux_response_interpolate(fluxobj->response->table, 9000., NULL,
                                              MUSE_FLUX_RESP_FLUX);
   cpl_msg_debug(__func__, "responses computed: %f ... %f ... %f", r1, r2, r3);
   cpl_test(fabs(r1 - 41.676) < 0.02 &&
            fabs(r2 - 41.581) < 0.02 &&
            fabs(r3 - 41.422) < 0.02);
+
+  /* test muse_flux_compute_qc() for the same dataset, now we can check *
+   * that we get at least similar values in the output header as the    *
+   * values from the unsmoothed version before;                         *
+   * take this chance to test the case of (partially) invalid rows      */
+  state = cpl_errorstate_get();
+  cpl_table_set_invalid(fluxobj->response->table, "lambda", 10);
+  cpl_table_set_invalid(fluxobj->response->table, "response", 12);
+  rc = muse_flux_compute_qc(fluxobj);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  const char *starname = cpl_propertylist_get_string(fluxobj->response->header,
+                                                     "ESO QC STANDARD STARNAME");
+  cpl_test_eq_string(starname, "GD71");
+  cpl_propertylist_erase_regexp(fluxobj->response->header,
+                                "ESO QC STANDARD THRU", 1);
+  cpl_test_eq(cpl_propertylist_get_size(fluxobj->response->header), 5);
+  float thru[] = { 0.3865, 0.2938, 0.2540, 0.1662, 0.1665 };
+  for (i = 0; i < 5; i++) {
+    char *kw = cpl_sprintf("ESO QC STANDARD THRU%04.0f", (i + 5.) * 1000.);
+    cpl_test_abs(cpl_propertylist_get_float(fluxobj->response->header, kw),
+                 thru[i], 0.01);
+    cpl_free(kw);
+  } /* for i (all 5 wavelengths) */
+  /* special case: OBS.TARG.NAME missing from cube header */
+  char *obstarg = cpl_strdup(muse_pfits_get_targname(fluxobj->cube->header));
+  cpl_propertylist_erase(fluxobj->cube->header, "ESO OBS TARG NAME");
+  state = cpl_errorstate_get();
+  rc = muse_flux_compute_qc(fluxobj);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  cpl_test_eq_string(cpl_propertylist_get_string(fluxobj->response->header,
+                                                 QC_STD_NAME), "UNKNOWN");
+  cpl_propertylist_append_string(fluxobj->cube->header, "ESO OBS TARG NAME",
+                                 obstarg);
+  cpl_free(obstarg);
+  /* test zeropoint calculation */
+  state = cpl_errorstate_get();
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, "Cousins_R");
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  float zp = cpl_propertylist_get_float(fluxobj->response->header,
+                                        "ESO QC STANDARD ZP R");
+  /* the INM apparently simulated a nice throughput of 27.3% in the R-band */
+  cpl_test(isfinite(zp));
+  cpl_test_rel(zp, 1.41028, 1e-5);
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, NULL);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  float zp2 = cpl_propertylist_get_float(fluxobj->response->header,
+                                         "ESO QC STANDARD ZP UNKNOWN");
+  cpl_test_eq(zp, zp2);
+  /* name should not matter */
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, "SDSS_g");
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  zp2 = cpl_propertylist_get_float(fluxobj->response->header,
+                                   "ESO QC STANDARD ZP g");
+  cpl_test_eq(zp, zp2);
+  /* name without underscore */
+  cpl_propertylist_erase(fluxobj->response->header,
+                         "ESO QC STANDARD ZP UNKNOWN");
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, "CousinsR");
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  zp2 = cpl_propertylist_get_float(fluxobj->response->header,
+                                   "ESO QC STANDARD ZP UNKNOWN");
+  cpl_test_eq(zp, zp2);
+  /* failure cases of muse_flux_compute_qc_zp() */
+  state = cpl_errorstate_get();
+  rc = muse_flux_compute_qc_zp(NULL, cousinsr, "Cousins_R");
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_table_delete(fluxobj->reference);
+  fluxobj->reference = NULL;
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, "Cousins_R");
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  fluxobj->reference = cpl_table_duplicate(ref);
+  /* test when both sensitivity and response curve are missing */
+  cpl_table *senssave = fluxobj->sensitivity;
+  muse_table *respsave = fluxobj->response;
+  fluxobj->sensitivity = NULL;
+  fluxobj->response = NULL;
+  rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, "Cousins_R");
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  fluxobj->sensitivity = senssave;
+  fluxobj->response = respsave;
+  rc = muse_flux_compute_qc_zp(fluxobj, NULL, "Cousins_R");
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  muse_table_delete(cousinsr);
+
+  /* check muse_flux_get_telluric_table() */
   rc = muse_flux_get_telluric_table(fluxobj);
   cpl_test_nonnull(fluxobj->telluric);
-  cpl_test(cpl_table_has_column(fluxobj->telluric, "lambda") &&
-           cpl_table_has_column(fluxobj->telluric, "ftelluric") &&
-           cpl_table_has_column(fluxobj->telluric, "ftellerr"));
-  cpl_test(cpl_table_get_nrow(fluxobj->telluric)
+  cpl_test_nonnull(fluxobj->telluric->table);
+  cpl_test_nonnull(fluxobj->telluric->header);
+  cpl_test(cpl_table_has_column(fluxobj->telluric->table, "lambda") &&
+           cpl_table_has_column(fluxobj->telluric->table, "ftelluric") &&
+           cpl_table_has_column(fluxobj->telluric->table, "ftellerr"));
+  cpl_test(cpl_table_get_nrow(fluxobj->telluric->table)
            < cpl_table_get_nrow(fluxobj->sensitivity));
-  double mean = cpl_table_get_column_mean(fluxobj->telluric, "ftelluric"),
-         median = cpl_table_get_column_median(fluxobj->telluric, "ftelluric"),
-         stdev = cpl_table_get_column_stdev(fluxobj->telluric, "ftelluric");
+  double mean = cpl_table_get_column_mean(fluxobj->telluric->table, "ftelluric"),
+         median = cpl_table_get_column_median(fluxobj->telluric->table, "ftelluric"),
+         stdev = cpl_table_get_column_stdev(fluxobj->telluric->table, "ftelluric");
   cpl_msg_debug(__func__, "telluric correction statistics: %f (%f) +/- %f",
                 mean, median, stdev);
   cpl_test(fabs(mean - 1.) < 0.092 && fabs(median - 1.) < 0.032 &&
@@ -600,12 +787,16 @@ int main(int argc, char **argv)
   cpl_table_delete(fluxobj->tellbands);
   fluxobj->sensitivity = NULL;
   fluxobj->tellbands = NULL;
+  /* also clean up the reference table for the next try */
+  cpl_table_delete(fluxobj->reference);
+  fluxobj->reference = NULL;
   state = cpl_errorstate_get();
   fluxobj->raref = 15.8871528; /* RA of GD71 */
   fluxobj->decref = 88.1150583; /* DEC of GD71 */
   rc = muse_flux_response_compute(fluxobj, MUSE_FLUX_SELECT_NEAREST, airmass,
                                   ref, NULL, extinction);
   cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+  cpl_test_nonnull(fluxobj->reference);
   cpl_test_nonnull(fluxobj->sensitivity);
   cpl_test_zero(cpl_table_compare_structure(sens1, fluxobj->sensitivity));
   /* compare both tables row by row */
@@ -670,7 +861,8 @@ int main(int argc, char **argv)
 
   cpl_msg_info(__func__, "using pixel table with %"CPL_SIZE_FORMAT" rows, "
                "response table with %"CPL_SIZE_FORMAT" entries",
-               muse_pixtable_get_nrow(pt), cpl_table_get_nrow(fluxobj->response));
+               muse_pixtable_get_nrow(pt),
+               cpl_table_get_nrow(fluxobj->response->table));
 
   /* duplicate the relevant columns to be able to compare before/after */
   cpl_table_duplicate_column(pt->table, "data_orig",
@@ -698,31 +890,67 @@ int main(int argc, char **argv)
   cpl_test(muse_flux_response_compute(fo, MUSE_FLUX_SELECT_BRIGHTEST, 1.0, ref,
                                       NULL, NULL) == CPL_ERROR_NONE);
   cpl_test(muse_flux_get_response_table(fo, MUSE_FLUX_SMOOTH_PPOLY) == CPL_ERROR_NONE);
-
+  cpl_test_nonnull(fo->response);
 #if 0
-  cpl_error_code ec = cpl_plot_column(NULL, NULL, NULL, fo->response, "lambda",
-                                      "response");
+  cpl_error_code ec = cpl_plot_column(NULL, NULL, NULL, fo->response->table,
+                                      "lambda", "response");
   printf("plotted: %d\n\n", ec); fflush(stdout);
 #endif
   /* do not count the extrapolated regions */
-  cpl_table_select_all(fo->response);
-  cpl_table_and_selected_double(fo->response, "lambda", CPL_LESS_THAN, 4800.);
-  cpl_table_or_selected_double(fo->response, "lambda", CPL_GREATER_THAN, 9300.);
-  cpl_table_erase_selected(fo->response);
+  cpl_table_select_all(fo->response->table);
+  cpl_table_and_selected_double(fo->response->table, "lambda", CPL_LESS_THAN, 4800.);
+  cpl_table_or_selected_double(fo->response->table, "lambda", CPL_GREATER_THAN, 9300.);
+  cpl_table_erase_selected(fo->response->table);
 #if 0
-  ec = cpl_plot_column(NULL, NULL, NULL, fo->response, "lambda", "response");
+  ec = cpl_plot_column(NULL, NULL, NULL, fo->response->table, "lambda",
+                       "response");
   printf("plotted: %d\n\n", ec); fflush(stdout);
 #endif
-  mean = cpl_table_get_column_mean(fo->response, "response");
-  median = cpl_table_get_column_median(fo->response, "response");
-  stdev = cpl_table_get_column_stdev(fo->response, "response");
-  double min = cpl_table_get_column_min(fo->response, "response"),
-         max = cpl_table_get_column_max(fo->response, "response");
+  mean = cpl_table_get_column_mean(fo->response->table, "response");
+  median = cpl_table_get_column_median(fo->response->table, "response");
+  stdev = cpl_table_get_column_stdev(fo->response->table, "response");
+  double min = cpl_table_get_column_min(fo->response->table, "response"),
+         max = cpl_table_get_column_max(fo->response->table, "response");
   cpl_msg_debug(__func__, "test-calibration statistics: %f (%f) +/- %f, %f ... %f",
                 mean, median, stdev, min, max);
   cpl_test(fabs(mean - 43.55) < 0.1); /* XXX why is it not ~50 mag == 1e-20 erg/s/cm**2/Angstrom? */
   cpl_test(fabs(mean - median) < 0.01 && stdev < 0.015);
   cpl_test(max - min < 0.12);
+  /* test the same response again, without smoothing this time! */
+  muse_table_delete(fo->response);
+  fo->response = NULL;
+  cpl_table_erase_window(fo->sensitivity, 50, 5);
+  cpl_test(muse_flux_get_response_table(fo, MUSE_FLUX_SMOOTH_NONE) == CPL_ERROR_NONE);
+  cpl_test_nonnull(fo->response);
+  /* results should be roughly similar */
+  mean = cpl_table_get_column_mean(fo->response->table, "response");
+  median = cpl_table_get_column_median(fo->response->table, "response");
+  stdev = cpl_table_get_column_stdev(fo->response->table, "response");
+  min = cpl_table_get_column_min(fo->response->table, "response");
+  max = cpl_table_get_column_max(fo->response->table, "response");
+  cpl_msg_debug(__func__, "test-calibration statistics (unsmoothed): "
+                "%f (%f) +/- %f, %f ... %f", mean, median, stdev, min, max);
+  cpl_test(fabs(mean - 43.55) < 0.1);
+  cpl_test(fabs(mean - median) < 0.01 && stdev < 0.02);
+  cpl_test(max - min < 0.25);
+  /* test the same response and smoothing again, but with a few rows removed to *
+   * trigger the jump-handling in muse_flux_get_response_table_piecewise_poly() */
+  muse_table_delete(fo->response);
+  fo->response = NULL;
+  cpl_table_erase_window(fo->sensitivity, 50, 5);
+  cpl_test(muse_flux_get_response_table(fo, MUSE_FLUX_SMOOTH_PPOLY) == CPL_ERROR_NONE);
+  cpl_test_nonnull(fo->response);
+  /* results should be very similar to the first case */
+  mean = cpl_table_get_column_mean(fo->response->table, "response");
+  median = cpl_table_get_column_median(fo->response->table, "response");
+  stdev = cpl_table_get_column_stdev(fo->response->table, "response");
+  min = cpl_table_get_column_min(fo->response->table, "response");
+  max = cpl_table_get_column_max(fo->response->table, "response");
+  cpl_msg_debug(__func__, "test-calibration statistics (missing rows): "
+                "%f (%f) +/- %f, %f ... %f", mean, median, stdev, min, max);
+  cpl_test(fabs(mean - 43.55) < 0.1);
+  cpl_test(fabs(mean - median) < 0.01 && stdev < 0.015);
+  cpl_test(max - min < 0.12);
   muse_flux_object_delete(fo);
   cpl_table_delete(ref);
 
